/*!
* @file libharkio3_log.c
*
* @author Takeshi Mizumoto
* @version 0.1
* @brief  harkio_Log の API群。 エラーやデバッグメッセージを表示する。
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "libharkio3.h"
#include "libharkio3_macros.h"

// LogLevelには、メッセージのログレベルをセットする
// PrintLevelには、コマンド引数で設定されたログレベルをセットする
void harkio_Log_printf_(enum harkio_Log_logLevel LogLevel, char *message , int PrintLevel, const char *func, int line)
{
    static int level = harkio_Log_Error;

    //harkio_Log_setLevel only
    if(LogLevel == -1) {
        level = PrintLevel;
        return;
    }


    switch(level) {
    case harkio_Log_Debug:
        if( LogLevel == harkio_Log_Debug ) {
            LIBHARKIO_LOGD("[D] [%s] [%s] [%d]\n", message , func , line);
        }
        else if( LogLevel == harkio_Log_Warning ) {
            LIBHARKIO_LOGE("[W] [%s] [%s] [%d]\n", message , func , line);
        }
        else if( LogLevel == harkio_Log_Error ) {
            LIBHARKIO_LOGE("[E] [%s] [%s] [%d]\n", message , func , line);
        }
        break;

    case harkio_Log_Warning:
        if( LogLevel == harkio_Log_Warning ) {
            LIBHARKIO_LOGE("[W] [%s] [%s] [%d]\n", message , func , line);
        }
        else if( LogLevel == harkio_Log_Error ) {
            LIBHARKIO_LOGE("[E] [%s] [%s] [%d]\n", message , func , line);
        }
        break;

    case harkio_Log_Info:
        if( LogLevel == harkio_Log_Info ) {
            LIBHARKIO_LOGI("[I] [%s] [%s] [%d]\n", message , func , line);
        }
        else if( LogLevel == harkio_Log_Debug ) {
            LIBHARKIO_LOGD("[D] [%s] [%s] [%d]\n", message , func , line);
        }
        else if( LogLevel == harkio_Log_Warning ) {
            LIBHARKIO_LOGE("[W] [%s] [%s] [%d]\n", message , func , line);
        }
        else if( LogLevel == harkio_Log_Error ) {
            LIBHARKIO_LOGE("[E] [%s] [%s] [%d]\n", message , func , line);
        }
        break;

    default:
        if( LogLevel == harkio_Log_Error ) {
            LIBHARKIO_LOGE("[E] [%s] [%s] [%d]\n", message , func , line);
        }
        break;

    }

}




// supporting functions
void harkio_Log_FileOpenFailed(char *filename)
{
    // for file open failed
    char *constmsg = " File open failed: ";
    char *msg = (char *) malloc(sizeof(char) * (
        strlen(filename) + strlen(constmsg)) + 2);

    sprintf(msg, "%s%s", constmsg, filename);
    harkio_Log_printf(harkio_Log_Error, msg);
    free(msg);
}

void harkio_Log_WrongValue(char *valuename, char *reason){
    // for wrong value
    char *constmsg = " has a wrong value: ";
    char *msg = (char *) malloc(sizeof(char) * (
         strlen(valuename) + strlen(reason) + strlen(constmsg)) + 2);

    sprintf(msg, "%s%s%s", valuename, constmsg, reason);
    harkio_Log_printf(harkio_Log_Error, msg);
    free(msg);
}

void harkio_Log_NullPointer(char *valuename){
    // for null pointers
    char *constmsg = " is NULL";
    char *msg = (char *) malloc(sizeof(char) * (
        strlen(valuename) + strlen(constmsg)) + 2);

    sprintf(msg, "%s%s", valuename, constmsg);
    harkio_Log_printf(harkio_Log_Error, msg);
    free(msg);
}






